@@ -35,6 +35,10 @@ from the X Consortium.
 ** Contains the bare minimum necessary to oversee the whole operation.
 */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Xaw/Form.h>
@@ -79,7 +83,7 @@ static XgcData FunctionData[NUM_FUNCTIONS] = {
   {"orInverted",   "orInverted",   GXorInverted},
   {"nand",         "nand",         GXnand},
   {"set",          "set",          GXset}
-}; 
+};
 
 /* The two rows in the XgcStuff structure are:
 **   name of label, xgc syntax text, # of toggles, # of columns of toggles
@@ -252,7 +256,7 @@ main(int argc, char *argv[])
     {XtNinput, 	      (XtArgVal) True}
   };
 
-  static Arg testformargs[] = {	
+  static Arg testformargs[] = {
     {XtNfromVert,     (XtArgVal) NULL} /* put it under GCform */
   };
 
@@ -284,6 +288,13 @@ main(int argc, char *argv[])
     {XtNborderWidth, (XtArgVal) 0}
   };
 
+  static const char * const usage =
+    "Usage: xgc [-options ...]\n"
+    "\n"
+    "where options include all standard toolkit options plus:\n"
+    "    -help       Print usage message and exit\n"
+    "    -version    Print version and exit\n";
+
   int i;			/* counter */
 
   /* Open the pipe */
@@ -297,9 +308,36 @@ main(int argc, char *argv[])
 
   XtSetLanguageProc(NULL, (XtLanguageProc) NULL, NULL);
 
+  /* Handle args that don't require opening a display */
+  for (int a = 1; a < argc; a++) {
+    const char *argn = argv[a];
+    /* accept single or double dash for -help & -version */
+    if (argn[0] == '-' && argn[1] == '-') {
+      argn++;
+    }
+    if (strcmp(argn, "-help") == 0) {
+      fputs(usage, stdout);
+      exit(EXIT_SUCCESS);
+    }
+    if (strcmp(argn, "-version") == 0) {
+      puts(PACKAGE_STRING);
+      exit(EXIT_SUCCESS);
+    }
+  }
+
   bigdaddy = XtAppInitialize(&appcontext, "Xgc", (XrmOptionDescList) NULL,
 			     (Cardinal) 0, &argc, argv, (String *) NULL,
 			     shellargs, XtNumber(shellargs));
+  if (argc > 1) {
+    fputs("Unrecognized argument(s):", stderr);
+    for (int a = 1; a < argc; a++) {
+      fprintf(stderr, " %s", argv[a]);
+    }
+    fputs("\n\n", stderr);
+    fputs(usage, stderr);
+    exit(EXIT_FAILURE);
+  }
+
   X.dpy = XtDisplay(bigdaddy);
   XtAppAddActions(appcontext, actions, XtNumber(actions));
   XtOverrideTranslations
@@ -377,7 +415,7 @@ main(int argc, char *argv[])
   backgroundchoice = XtCreateManagedWidget("background",formWidgetClass,GCform,
 				   gcchoiceargs,XtNumber(gcchoiceargs));
   backgroundtext = create_text_choice(backgroundchoice,TBackground,9,50);
-  
+
   gcchoiceargs[1].value = (XtArgVal) NULL;
   gcchoiceargs[0].value = (XtArgVal) foregroundchoice;
   percentchoice = XtCreateManagedWidget("testpercent",formWidgetClass,GCform,
@@ -392,7 +430,7 @@ main(int argc, char *argv[])
   testformargs[0].value = (XtArgVal) GCform;
   Testform = XtCreateManagedWidget("Testform",formWidgetClass,topform,
 				   testformargs,XtNumber(testformargs));
-  
+
   testchoiceform = XtCreateManagedWidget("testchoiceform",formWidgetClass,
 			     Testform,testchoiceargs,XtNumber(testchoiceargs));
   testchoicedesc = create_choice(testchoiceform,Everything[CTest]);
@@ -445,7 +483,7 @@ main(int argc, char *argv[])
 
   choose_defaults(GCdescs,(int)XtNumber(GCdescs));
   choose_defaults(&testchoicedesc,1);
-  
+
   /* Loop forever, dealing with events */
 
   XtAppMainLoop(appcontext);
@@ -553,8 +591,8 @@ fill_up_commandform(Widget w)
   quitargs[1].value = (XtArgVal) keyinputbutton; /* under */
   (void) XtCreateManagedWidget("Quit",commandWidgetClass,
 			       w,quitargs,XtNumber(quitargs));
-    
-}    
+
+}
 /* quit()
 ** ------
 ** Leave the program nicely.
@@ -571,7 +609,7 @@ static void quitAction(Widget w, XEvent *e, String *p, Cardinal *n)
 {
     if (e->type == ClientMessage && e->xclient.data.l[0] != wm_delete_window)
 	XBell(XtDisplay(w), 0);
-    else 
+    else
 	quit();
 }
 
